package fi.jyu.intellij.comtest.rider

import com.intellij.openapi.project.Project
import comtest.v2.csharp.ComTestNUnitBackend
import comtest.v2.processors.*
import fi.jyu.intellij.comtest.common.util.Notifier
import fi.jyu.intellij.comtest.rider.util.riderSolutionFile

/**
 * Wrapper object for ComTest (v2).
 *
 * For now, only supports generating NUnit tests.
 */
internal object ComTestRunner {
    init {
        val backend = ComTestNUnitBackend()
        FileProcessor.register(Language.Csharp, CsFileProcessor(backend, ComTestLoggerWindow.ComTestLogger))
        FileProcessor.register(Language.CsharpCore, CsFileProcessor(backend, ComTestLoggerWindow.ComTestLogger))
        FileProcessor.register(Language.CsharpProject, CsProjectProcessor(backend, ComTestLoggerWindow.ComTestLogger))
        FileProcessor.register(Language.CsharpSolution, CsSolutionProcessor(backend, ComTestLoggerWindow.ComTestLogger))
    }

    /**
     * Generate tests for .NET solution.
     *
     * @param proj Solution for which to generate the tests.
     */
    fun generateForDotNetSln(proj: Project): ComTestResult? {
        ComTestLoggerWindow.open(proj)

        val slnFile = proj.riderSolutionFile
            ?: run {
                Notifier.notifyError(
                    proj,
                    "No solution found or solution is virtual (virtual solutions are not supported). Load a solution to generate tests."
                )
                return null
            }

        val slnEntry = FileEntry(slnFile.path, Language.CsharpSolution)
        return FileProcessor.process(slnEntry, slnFile.parent, false)
    }
}