package fi.jyu.intellij.comtest.rider.actions

import com.intellij.openapi.actionSystem.ActionManager
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.AnActionEvent
import com.jetbrains.rider.build.BuildHost
import com.jetbrains.rider.build.BuildParameters
import com.jetbrains.rider.model.BuildTarget
import comtest.v2.processors.ComTestResult
import fi.jyu.intellij.comtest.common.util.runWithRefresh
import fi.jyu.intellij.comtest.rider.ComTestRunner
import fi.jyu.intellij.comtest.rider.util.hasRiderSolution
import fi.jyu.intellij.comtest.rider.util.onSolutionAnalysisReadyFireOnce

/**
 * Action to run ComTest on the current .NET solution.
 */
@Suppress("MissingActionUpdateThread", "ComponentNotRegistered")
class ComTestGenerateDotNetSolutionAction : AnAction() {
    override fun actionPerformed(e: AnActionEvent) {
        val proj = e.project ?: return

        val res = runWithRefresh(proj) {
            ComTestRunner.generateForDotNetSln(proj)
        }

        if (res == null || (res !is ComTestResult.Success && res !is ComTestResult.SkippedAsGood)) {
            return
        }


        // csproj change will trigger assembly analysis
        // Wait for it to finish and then trigger unit test discovery
        proj.onSolutionAnalysisReadyFireOnce {
            // To discover tests, Rider needs to build them at least once
            BuildHost.getInstance(proj).requestBuild(
                BuildParameters(
                    BuildTarget(),
                    emptyList(),
                    diagnosticsMode = false,
                    silentMode = true,
                    withoutDependencies = false,
                    noRestore = false
                ),
            )
            // Just waiting for build to finish isn't enough, we need to wait for Rider test discovery to finish
            proj.onSolutionAnalysisReadyFireOnce {
                val act = ActionManager.getInstance().getAction("RiderUnitTestRunSolutionAction")!!
                act.update(e)
                act.actionPerformed(e)
            }
        }
    }

    override fun update(e: AnActionEvent) {
        e.presentation.isEnabled = e.project?.hasRiderSolution ?: false
    }
}